/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * @file doca_ct.h
 * @page doca ct
 * @defgroup Flow ct
 * DOCA HW connection tracking library.
 *
 * @{
 */

#ifndef DOCA_CT_H_
#define DOCA_CT_H_

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

#include <doca_compat.h>
#include <doca_error.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief CT flags
 */
enum doca_ct_types {
	DOCA_CT_TYPE_KNOWN, /**< Known TCP/UDP connection */
	DOCA_CT_TYPE_SYN, /**< New connection packet */
	DOCA_CT_TYPE_FIN, /**< Connection finish packet */
	DOCA_CT_TYPE_RST, /**< Connection reset packet */
};

/**
 * @brief CT flags
 */
enum doca_ct_flags {
	DOCA_CT_FLAG_STATS = 1u << 0, /**< Enable counter for internal pipes */
	DOCA_CT_FLAG_WORKER_STATS = 1u << 1, /**< Enable worker counter dump */
	DOCA_CT_FLAG_NO_AGING = 1u << 2, /**< Bypass aging scan */
};

/**
 * @brief CT l3 session types
 */
enum doca_ct_session_type {
	DOCA_CT_SESSION_IPV4,		/**< IPv4 session. */
	DOCA_CT_SESSION_IPV6,		/**< IPv6 session. */
	DOCA_CT_SESSION_BOTH,		/**< Total session. */
	DOCA_CT_SESSION_MAX,		/**< Max session types. */
};

/**
 * @brief doca ct global configuration
 */
struct doca_ct_cfg {
	uint32_t nb_arm_queues; /**< number of ARM CT queues(thread). */
	uint32_t nb_arm_sessions[DOCA_CT_SESSION_MAX]; /**< number of ARM CT sessions. */
	uint32_t flags; /**< CT behavior flags */
	void *ib_dev; /**< IB verbs device context */
	void *ib_pd; /**< device protection domain */
	uint16_t tcp_timeout_s; /**< TCP timeout in second. */
	uint16_t tcp_session_del_s; /**< time to kill TCP session after RST/FIN. */
	uint16_t udp_timeout_s; /**< UDP timeout in second. */
	uint16_t aging_core; /**< CT aging thread bind to CPU core. */
};

/**
 * @brief Initialize the doca ct.
 *
 * This is the global initialization function for doca ct. It
 * initializes all resources used by doca flow.
 *
 * Must be invoked first before any other function in this API.
 * this is a one time call, used for doca ct initialization and
 * global configurations.
 *
 * Must be invoked after Doca Flow initilization, before port start.
 *
 * @param cfg
 *   CT configuration.
 * @return
 *   0 on success, a negative errno value otherwise.
 */
__DOCA_EXPERIMENTAL
doca_error_t
doca_ct_init(const struct doca_ct_cfg *cfg);

/**
 * @brief Destroy the doca ct.
 *
 * Release all the resources used by doca ct.
 *
 * Must be invoked before doca flow detroy.
 */
__DOCA_EXPERIMENTAL
void
doca_ct_destroy(void);


#ifdef __cplusplus
} /* extern "C" */
#endif

/** @} */

#endif /* DOCA_CT_H_ */
