/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * @file doca_hotplug.h
 * @page doca hotplug
 * @defgroup HOTPLUG DOCA Hotplug
 * @ingroup DOCACore
 * DOCA API for hot plug/un-plug devices.
 *
 * @{
 */

#ifndef DOCA_HOTPLUG_H_
#define DOCA_HOTPLUG_H_

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

#include <doca_dev.h>
#include <doca_compat.h>
#include <doca_error.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Opaque structure representing the characteristics of a future representor
 * DOCA device that will be created using hotplug mechanism. This structure is associated with
 * a specific DOCA device.
 */
struct doca_dev_hotplug_attr;

/**
 * @brief Hotplug and initialize representor device for use.
 *
 * @param [in] attr
 * DOCA hotplug attr with designated characteristics.
 * @param [out] dev_rep
 * Initialized representor doca device instance on success. Valid on success only.
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * DOCA_ERROR_INVALID_VALUE - in case of invalid input.
 * DOCA_ERROR_NOT_SUPPORTED - in case of one of non hotplug manager device or unsupported emulated device type.
 * DOCA_ERROR_NOT_FOUND - in case the hotplugged device was exposed to the host PCI but its representor on the DPU couldn't be found.
 */
__DOCA_EXPERIMENTAL
doca_error_t doca_dev_rep_hotplug(const struct doca_dev_hotplug_attr *attr,
				     struct doca_dev_rep **dev_rep);

/**
 * @brief Destroy and unplug representor device instance.
 *
 * @param [in] rep_dev
 * The previously hotplugged representor doca device instance.
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * DOCA_ERROR_INVALID_VALUE - in case of invalid input.
 * DOCA_ERROR_NOT_SUPPORTED - in case of static emulated representor device.
 * @note For virtio representor devices it's recommended (due to a bug in Linux virtio drivers) to
 * destroy a controller with a special preparation for hotunplug operation prior calling this function.
 * See DOCA virtio documentation for more details.
 */
__DOCA_EXPERIMENTAL
doca_error_t doca_dev_rep_hotunplug(struct doca_dev_rep *rep_dev);

#ifdef __cplusplus
} /* extern "C" */
#endif

/** @} */

#endif  /* DOCA_HOTPLUG_H_ */
