/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * @file doca_types.h
 * @page doca types
 * @defgroup TYPES DOCA Types
 * @ingroup DOCACore
 * DOCA Types introduces types that are common for many libraries.
 *
 * @{
 */

#ifndef DOCA_TYPES_H_
#define DOCA_TYPES_H_

#include <stdint.h>

#ifdef __linux__
	#include <linux/types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __linux__
	typedef int doca_event_handle_t; /* 'fd' for blocking with epoll/select/poll, event type will be "read ready" */
	#define doca_event_invalid_handle -1
#else /* Windows*/
	typedef void *doca_event_handle_t; /* Used for windows HANDLE or IOCompletionPort */
	#define doca_event_invalid_handle INVALID_HANDLE_VALUE
#endif

union doca_data {
	void *ptr;
	uint64_t u64;
};

/**
 * @brief Specifies the permission level for DOCA buffer
 *
 * @details Can be used with doca_mmap_set_permissions() to set permission level. A few notes:
 * DOCA_ACCESS_DPU_READ_ONLY and DOCA_ACCESS_DPU_READ_WRITE are mutually exclusive
 * Buffer can always be read locally by local device, regardless of set permissions
 * local device - doca_dev running in the same process of the mmap
 * remote device - doca_dev running on a different process on a remote machine
 * DPU device - doca_dev running on a process on the DPU OS. This is only relevant when local process is running on
 * HOST. In case local process is running on DPU the doca_dev is considered a local device.
 */
enum doca_access_flags {
	DOCA_ACCESS_LOCAL_READ_ONLY	= 0,		/* Allows reading buffer by a local device but no write */
	DOCA_ACCESS_LOCAL_READ_WRITE	= (1 << 0),	/* Allows reading and writing to buffer by a local device */
	DOCA_ACCESS_RDMA_READ		= (1 << 1),	/* Allows reading buffer by a remote device using RDMA */
	DOCA_ACCESS_RDMA_WRITE		= (1 << 2),	/* Allows writing to buffer by a remote device using RDMA */
	DOCA_ACCESS_RDMA_ATOMIC		= (1 << 3),	/* Allows atomic operations by remote device using RDMA */
	DOCA_ACCESS_DPU_READ_ONLY	= (1 << 4),	/**
							 * Allows reading buffer by a DPU device but no write
							 * Only relevant for HOST - see doca_mmap_export_dpu()
							 */
	DOCA_ACCESS_DPU_READ_WRITE	= (1 << 5),	/**
							 * Allows reading and writing to buffer by a DPU device
							 * Only relevant for HOST - see doca_mmap_export_dpu()
							 */
};

/**
 * @brief Specifies the PCI function type for DOCA representor device
 *
 */
enum doca_pci_func_type {
	DOCA_PCI_FUNC_PF = 0, /* physical function */
	DOCA_PCI_FUNC_VF, /* virtual function */
	DOCA_PCI_FUNC_SF, /* sub function */
};

/**
 * @brief The PCI address of a device - same as the address in lspci
 *
 */
struct doca_pci_bdf {
	#define PCI_FUNCTION_MAX_VALUE 8
	#define PCI_DEVICE_MAX_VALUE 32
	#define PCI_BUS_MAX_VALUE 256
	union {
		uint16_t raw;
		struct {
			uint16_t function : 3;
			uint16_t device : 5;
			uint16_t bus : 8;
		};
	};
};


/**
 * @brief Declare DOCA endianity types
 *
 */
#ifdef __linux__
	typedef __be16 doca_be16_t;
	typedef __be32 doca_be32_t;
	typedef __be64 doca_be64_t;
#else /* Windows */
	typedef uint16_t doca_be16_t;
	typedef uint32_t doca_be32_t;
	typedef uint64_t doca_be64_t;
#endif

/**
 * @brief Type of memory the GPUNetIO library can allocate
 *
 */
enum doca_gpu_mem_type {
	/* GPU memory not accessible from CPU. */
	DOCA_GPU_MEM_GPU	= 0,
	/* GPU memory with direct access from CPU. */
	DOCA_GPU_MEM_GPU_CPU	= 1,
	/* CPU memory not accessible from GPU. */
	DOCA_GPU_MEM_CPU	= 2,
	/* CPU memory with direct access from GPU. */
	DOCA_GPU_MEM_CPU_GPU	= 3,
};


/**
 * @brief Type of wait on time the network card can support
 *
 */
enum doca_eth_wait_on_time {
	/* Wait on time not supported. */
	DOCA_ETH_WAIT_ON_TIME_NONE	= 0,
	/* Wait on time supported by HW. */
	DOCA_ETH_WAIT_ON_TIME_NATIVE	= 1,
	/* Wait on time supported by SW. */
	DOCA_ETH_WAIT_ON_TIME_DPDK	= 2,
};


#ifdef __cplusplus
}
#endif

/** @} */

#endif /* DOCA_TYPES_H_ */
